@@ -1,7 +1,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // detail/dynamic/parser_traits.hpp
 //
-//  Copyright 2007 Eric Niebler. Distributed under the Boost
+//  Copyright 2008 Eric Niebler. Distributed under the Boost
 //  Software License, Version 1.0. (See accompanying file
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -115,13 +115,13 @@ struct compiler_traits
         case BOOST_XPR_CHAR_(char_type, ')'): ++begin; return token_group_end;
         case BOOST_XPR_CHAR_(char_type, '|'): ++begin; return token_alternate;
         case BOOST_XPR_CHAR_(char_type, '['): ++begin; return token_charset_begin;
-        case BOOST_XPR_CHAR_(char_type, ']'): ++begin; return token_charset_end;
 
         case BOOST_XPR_CHAR_(char_type, '*'):
         case BOOST_XPR_CHAR_(char_type, '+'):
         case BOOST_XPR_CHAR_(char_type, '?'):
             return token_invalid_quantifier;
 
+        case BOOST_XPR_CHAR_(char_type, ']'):
         case BOOST_XPR_CHAR_(char_type, '{'):
         default:
             return token_literal;
@@ -299,10 +299,25 @@ struct compiler_traits
         case BOOST_XPR_CHAR_(char_type, '['):
             {
                 FwdIter next = begin; ++next;
-                if(next != end && *next == BOOST_XPR_CHAR_(char_type, ':'))
+                if(next != end)
                 {
-                    begin = ++next;
-                    return token_posix_charset_begin;
+                    detail::ensure(
+                        *next != BOOST_XPR_CHAR_(char_type, '=')
+                      , error_collate
+                      , "equivalence classes are not yet supported"
+                    );
+
+                    detail::ensure(
+                        *next != BOOST_XPR_CHAR_(char_type, '.')
+                      , error_collate
+                      , "collation sequences are not yet supported"
+                    );
+
+                    if(*next == BOOST_XPR_CHAR_(char_type, ':'))
+                    {
+                        begin = ++next;
+                        return token_posix_charset_begin;
+                    }
                 }
             }
             break;
