@@ -11,7 +11,7 @@
 
 //  See http://www.boost.org/libs/config for most recent version.
 //
-//  Revision $Id: config_info.cpp 41316 2007-11-23 17:03:14Z bemandawes $
+//  Revision $Id: config_info.cpp 49212 2008-10-09 16:08:03Z johnmaddock $
 //
 
 #include <boost/config.hpp>
@@ -952,6 +952,7 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_HAS_WINTHREADS);
    PRINT_MACRO(BOOST_MSVC6_MEMBER_TEMPLATES);
    PRINT_MACRO(BOOST_MSVC_STD_ITERATOR);
+   PRINT_MACRO(BOOST_NO_ADL_BARRIER);
    PRINT_MACRO(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP);
    PRINT_MACRO(BOOST_NO_ARRAY_TYPE_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_AUTO_PTR);
@@ -981,10 +982,13 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_FRIENDS);
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_KEYWORD);
    PRINT_MACRO(BOOST_NO_MS_INT64_NUMERIC_LIMITS);
+   PRINT_MACRO(BOOST_NO_NESTED_FRIENDSHIP);
    PRINT_MACRO(BOOST_NO_OPERATORS_IN_NAMESPACE);
+   PRINT_MACRO(BOOST_NO_PARTIAL_SPECIALIZATION_IMPLICIT_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_CONST);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_PRIVATE_IN_AGGREGATE);
+   PRINT_MACRO(BOOST_NO_RTTI);
    PRINT_MACRO(BOOST_NO_SFINAE);
    PRINT_MACRO(BOOST_NO_STDC_NAMESPACE);
    PRINT_MACRO(BOOST_NO_STD_ALLOCATOR);
@@ -1001,11 +1005,13 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_STD_WSTRING);
    PRINT_MACRO(BOOST_NO_STRINGSTREAM);
    PRINT_MACRO(BOOST_NO_SWPRINTF);
+   PRINT_MACRO(BOOST_NO_TEMPLATED_IOSTREAMS);
    PRINT_MACRO(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS);
    PRINT_MACRO(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION);
    PRINT_MACRO(BOOST_NO_TEMPLATE_TEMPLATES);
    PRINT_MACRO(BOOST_NO_TWO_PHASE_NAME_LOOKUP);
    PRINT_MACRO(BOOST_NO_TYPEID);
+   PRINT_MACRO(BOOST_NO_TYPENAME_WITH_CTOR);
    PRINT_MACRO(BOOST_NO_UNREACHABLE_RETURN_DETECTION);
    PRINT_MACRO(BOOST_NO_USING_DECLARATION_OVERLOADS_FROM_TYPENAME_BASE);
    PRINT_MACRO(BOOST_NO_USING_TEMPLATE);
@@ -1014,6 +1020,22 @@ void print_boost_macros()
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
