@@ -1,12 +1,21 @@
 //  path_test program  -------------------------------------------------------//
 
-//  Copyright Beman Dawes 2002.
+//  Copyright Beman Dawes 2002
+//  Copyright Vladimir Prus 2002
+
 //  Use, modification, and distribution is subject to the Boost Software
 //  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 //  http://www.boost.org/LICENSE_1_0.txt)
 
 //  See library home page at http://www.boost.org/libs/filesystem
 
+//  basic_path's stem(), extension(), and replace_extension() tests are based
+//  on basename(), extension(), and change_extension() tests from the original
+//  convenience_test.cpp by Vladimir Prus.
+
+//  See deprecated_test for tests of deprecated features
+#define BOOST_FILESYSTEM_NO_DEPRECATED
+
 #include <boost/filesystem/operations.hpp>
 #include <boost/utility.hpp>
 #include <iostream>
@@ -30,10 +39,10 @@ using boost::prior;
 
 namespace
 {
-  int errors;
-
   std::string platform( BOOST_PLATFORM );
 
+  int errors;
+
   void check( const fs::path & source,
               const std::string & expected, int line )
   {
@@ -71,111 +80,6 @@ namespace
               << "\"" << std::endl;
   }
 
-  void check_normalize()
-  {
-    PATH_CHECK( path("").normalize(), "" );
-    PATH_CHECK( path("/").normalize(), "/" );
-    PATH_CHECK( path("//").normalize(), "//" );
-    PATH_CHECK( path("///").normalize(), "/" );
-    PATH_CHECK( path("f").normalize(), "f" );
-    PATH_CHECK( path("foo").normalize(), "foo" );
-    PATH_CHECK( path("foo/").normalize(), "foo/." );
-    PATH_CHECK( path("f/").normalize(), "f/." );
-    PATH_CHECK( path( "/foo" ).normalize(), "/foo" );
-    PATH_CHECK( path( "foo/bar" ).normalize(), "foo/bar" );
-    PATH_CHECK( path("..").normalize(), ".." );
-    PATH_CHECK( path("../..").normalize(), "../.." );
-    PATH_CHECK( path("/..").normalize(), "/.." );
-    PATH_CHECK( path("/../..").normalize(), "/../.." );
-    PATH_CHECK( path("../foo").normalize(), "../foo" );
-    PATH_CHECK( path("foo/..").normalize(), "." );
-    PATH_CHECK( path("foo/../").normalize(), "./." );
-    PATH_CHECK( (path("foo") / "..").normalize() , "." );
-    PATH_CHECK( path("foo/...").normalize(), "foo/..." );
-    PATH_CHECK( path("foo/.../").normalize(), "foo/.../." );
-    PATH_CHECK( path("foo/..bar").normalize(), "foo/..bar" );
-    PATH_CHECK( path("../f").normalize(), "../f" );
-    PATH_CHECK( path("/../f").normalize(), "/../f" );
-    PATH_CHECK( path("f/..").normalize(), "." );
-    PATH_CHECK( (path("f") / "..").normalize() , "." );
-    PATH_CHECK( path("foo/../..").normalize(), ".." );
-    PATH_CHECK( path("foo/../../").normalize(), "../." );
-    PATH_CHECK( path("foo/../../..").normalize(), "../.." );
-    PATH_CHECK( path("foo/../../../").normalize(), "../../." );
-    PATH_CHECK( path("foo/../bar").normalize(), "bar" );
-    PATH_CHECK( path("foo/../bar/").normalize(), "bar/." );
-    PATH_CHECK( path("foo/bar/..").normalize(), "foo" );
-    PATH_CHECK( path("foo/bar/../").normalize(), "foo/." );
-    PATH_CHECK( path("foo/bar/../..").normalize(), "." );
-    PATH_CHECK( path("foo/bar/../../").normalize(), "./." );
-    PATH_CHECK( path("foo/bar/../blah").normalize(), "foo/blah" );
-    PATH_CHECK( path("f/../b").normalize(), "b" );
-    PATH_CHECK( path("f/b/..").normalize(), "f" );
-    PATH_CHECK( path("f/b/../").normalize(), "f/." );
-    PATH_CHECK( path("f/b/../a").normalize(), "f/a" );
-    PATH_CHECK( path("foo/bar/blah/../..").normalize(), "foo" );
-    PATH_CHECK( path("foo/bar/blah/../../bletch").normalize(), "foo/bletch" );
-    PATH_CHECK( path( "//net" ).normalize(), "//net" );
-    PATH_CHECK( path( "//net/" ).normalize(), "//net/" );
-    PATH_CHECK( path( "//..net" ).normalize(), "//..net" );
-    PATH_CHECK( path( "//net/.." ).normalize(), "//net/.." );
-    PATH_CHECK( path( "//net/foo" ).normalize(), "//net/foo" );
-    PATH_CHECK( path( "//net/foo/" ).normalize(), "//net/foo/." );
-    PATH_CHECK( path( "//net/foo/.." ).normalize(), "//net/" );
-    PATH_CHECK( path( "//net/foo/../" ).normalize(), "//net/." );
-
-    PATH_CHECK( path( "/net/foo/bar" ).normalize(), "/net/foo/bar" );
-    PATH_CHECK( path( "/net/foo/bar/" ).normalize(), "/net/foo/bar/." );
-    PATH_CHECK( path( "/net/foo/.." ).normalize(), "/net" );
-    PATH_CHECK( path( "/net/foo/../" ).normalize(), "/net/." );
-
-    PATH_CHECK( path( "//net//foo//bar" ).normalize(), "//net/foo/bar" );
-    PATH_CHECK( path( "//net//foo//bar//" ).normalize(), "//net/foo/bar/." );
-    PATH_CHECK( path( "//net//foo//.." ).normalize(), "//net/" );
-    PATH_CHECK( path( "//net//foo//..//" ).normalize(), "//net/." );
-
-    PATH_CHECK( path( "///net///foo///bar" ).normalize(), "/net/foo/bar" );
-    PATH_CHECK( path( "///net///foo///bar///" ).normalize(), "/net/foo/bar/." );
-    PATH_CHECK( path( "///net///foo///.." ).normalize(), "/net" );
-    PATH_CHECK( path( "///net///foo///..///" ).normalize(), "/net/." );
-
-    if ( platform == "Windows" )
-    {
-      PATH_CHECK( path( "c:.." ).normalize(), "c:.." );
-      PATH_CHECK( path( "c:foo/.." ).normalize(), "c:" );
-
-      PATH_CHECK( path( "c:foo/../" ).normalize(), "c:." );
-
-      PATH_CHECK( path( "c:/foo/.." ).normalize(), "c:/" );
-      PATH_CHECK( path( "c:/foo/../" ).normalize(), "c:/." );
-      PATH_CHECK( path( "c:/.." ).normalize(), "c:/.." );
-      PATH_CHECK( path( "c:/../" ).normalize(), "c:/../." );
-      PATH_CHECK( path( "c:/../.." ).normalize(), "c:/../.." );
-      PATH_CHECK( path( "c:/../../" ).normalize(), "c:/../../." );
-      PATH_CHECK( path( "c:/../foo" ).normalize(), "c:/../foo" );
-      PATH_CHECK( path( "c:/../foo/" ).normalize(), "c:/../foo/." );
-      PATH_CHECK( path( "c:/../../foo" ).normalize(), "c:/../../foo" );
-      PATH_CHECK( path( "c:/../../foo/" ).normalize(), "c:/../../foo/." );
-      PATH_CHECK( path( "c:/..foo" ).normalize(), "c:/..foo" );
-    }
-    else // POSIX
-    {
-      PATH_CHECK( path( "c:.." ).normalize(), "c:.." );
-      PATH_CHECK( path( "c:foo/.." ).normalize(), "." );
-      PATH_CHECK( path( "c:foo/../" ).normalize(), "./." );
-      PATH_CHECK( path( "c:/foo/.." ).normalize(), "c:" );
-      PATH_CHECK( path( "c:/foo/../" ).normalize(), "c:/." );
-      PATH_CHECK( path( "c:/.." ).normalize(), "." );
-      PATH_CHECK( path( "c:/../" ).normalize(), "./." );
-      PATH_CHECK( path( "c:/../.." ).normalize(), ".." );
-      PATH_CHECK( path( "c:/../../" ).normalize(), "../." );
-      PATH_CHECK( path( "c:/../foo" ).normalize(), "foo" );
-      PATH_CHECK( path( "c:/../foo/" ).normalize(), "foo/." );
-      PATH_CHECK( path( "c:/../../foo" ).normalize(), "../foo" );
-      PATH_CHECK( path( "c:/../../foo/" ).normalize(), "../foo/." );
-      PATH_CHECK( path( "c:/..foo" ).normalize(), "c:/..foo" );
-    }
-  }
 
   void exception_tests()
   {
@@ -248,6 +152,18 @@ int test_main( int, char*[] )
   path p5;
   std::string s1( "//:somestring" );
 
+  // verify deprecated names still available
+
+# ifndef BOOST_FILESYSTEM_NO_DEPRECATED
+
+  p1.branch_path();
+  p1.leaf();
+  path p_remove_leaf;
+  p_remove_leaf.remove_leaf();
+
+# endif
+
+
 # ifndef BOOST_NO_MEMBER_TEMPLATES
 
   // check the path member templates
@@ -290,10 +206,10 @@ int test_main( int, char*[] )
   BOOST_CHECK( p1 != p4 );
   BOOST_CHECK( p1.string() == p2.string() );
   BOOST_CHECK( p1.string() == p3.string() );
-  BOOST_CHECK( path( "foo" ).leaf() == "foo" );
-  BOOST_CHECK( path( "foo" ).branch_path().string() == "" );
-  BOOST_CHECK( p1.leaf() == "fum" );
-  BOOST_CHECK( p1.branch_path().string() == "fe/fi/fo" );
+  BOOST_CHECK( path( "foo" ).filename() == "foo" );
+  BOOST_CHECK( path( "foo" ).parent_path().string() == "" );
+  BOOST_CHECK( p1.filename() == "fum" );
+  BOOST_CHECK( p1.parent_path().string() == "fe/fi/fo" );
   BOOST_CHECK( path( "" ).empty() == true );
   BOOST_CHECK( path( "foo" ).empty() == false );
 
@@ -325,18 +241,16 @@ int test_main( int, char*[] )
     PATH_CHECK( path("c:") / "/foo", "c:/foo" );
   }
 
-  check_normalize();
-
   if ( platform == "Windows" )
   {
-    PATH_CHECK( path("c:", fs::native ) / "foo", "c:foo" );
-    PATH_CHECK( path("c:", fs::native) / "/foo", "c:/foo" );
+    PATH_CHECK( path("c:") / "foo", "c:foo" );
+    PATH_CHECK( path("c:") / "/foo", "c:/foo" );
   }
 
   PATH_CHECK( "foo/bar", "foo/bar" );
   PATH_CHECK( path("foo") / path("bar"), "foo/bar" ); // path arg
   PATH_CHECK( path("foo") / "bar", "foo/bar" );       // const char * arg
-  PATH_CHECK( path("foo") / path("woo/bar").leaf(), "foo/bar" ); // const std::string & arg
+  PATH_CHECK( path("foo") / path("woo/bar").filename(), "foo/bar" ); // const std::string & arg
   PATH_CHECK( "foo" / path("bar"), "foo/bar" );
 
   PATH_CHECK( "a/b", "a/b" );  // probe for length effects
@@ -616,8 +530,8 @@ int test_main( int, char*[] )
 
   p = "";
   BOOST_CHECK( p.relative_path().string() == "" );
-  BOOST_CHECK( p.branch_path().string() == "" );
-  BOOST_CHECK( p.leaf() == "" );
+  BOOST_CHECK( p.parent_path().string() == "" );
+  BOOST_CHECK( p.filename() == "" );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "" );
   BOOST_CHECK( p.root_path().string() == "" );
@@ -625,14 +539,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( !p.has_relative_path() );
-  BOOST_CHECK( !p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( !p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "/";
   BOOST_CHECK( p.relative_path().string() == "" );
-  BOOST_CHECK( p.branch_path().string() == "" );
-  BOOST_CHECK( p.leaf() == "/" );
+  BOOST_CHECK( p.parent_path().string() == "" );
+  BOOST_CHECK( p.filename() == "/" );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "/" );
   BOOST_CHECK( p.root_path().string() == "/" );
@@ -640,8 +554,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( !p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   if ( platform == "POSIX" )
     BOOST_CHECK( p.is_complete() );
   else
@@ -649,8 +563,8 @@ int test_main( int, char*[] )
 
   p = "//";
   CHECK_EQUAL( p.relative_path().string(), "" );
-  CHECK_EQUAL( p.branch_path().string(), "" );
-  CHECK_EQUAL( p.leaf(), "//" );
+  CHECK_EQUAL( p.parent_path().string(), "" );
+  CHECK_EQUAL( p.filename(), "//" );
   CHECK_EQUAL( p.root_name(), "//" );
   CHECK_EQUAL( p.root_directory(), "" );
   CHECK_EQUAL( p.root_path().string(), "//" );
@@ -658,15 +572,15 @@ int test_main( int, char*[] )
   BOOST_CHECK( p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( !p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
 
   p = "///";
   CHECK_EQUAL( p.relative_path().string(), "" );
-  CHECK_EQUAL( p.branch_path().string(), "" );
-  CHECK_EQUAL( p.leaf(), "/" );
+  CHECK_EQUAL( p.parent_path().string(), "" );
+  CHECK_EQUAL( p.filename(), "/" );
   CHECK_EQUAL( p.root_name(), "" );
   CHECK_EQUAL( p.root_directory(), "/" );
   CHECK_EQUAL( p.root_path().string(), "/" );
@@ -674,8 +588,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( !p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   if ( platform == "POSIX" )
     BOOST_CHECK( p.is_complete() );
   else
@@ -683,8 +597,8 @@ int test_main( int, char*[] )
 
   p = ".";
   BOOST_CHECK( p.relative_path().string() == "." );
-  BOOST_CHECK( p.branch_path().string() == "" );
-  BOOST_CHECK( p.leaf() == "." );
+  BOOST_CHECK( p.parent_path().string() == "" );
+  BOOST_CHECK( p.filename() == "." );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "" );
   BOOST_CHECK( p.root_path().string() == "" );
@@ -692,14 +606,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "..";
   BOOST_CHECK( p.relative_path().string() == ".." );
-  BOOST_CHECK( p.branch_path().string() == "" );
-  BOOST_CHECK( p.leaf() == ".." );
+  BOOST_CHECK( p.parent_path().string() == "" );
+  BOOST_CHECK( p.filename() == ".." );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "" );
   BOOST_CHECK( p.root_path().string() == "" );
@@ -707,14 +621,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "foo";
   BOOST_CHECK( p.relative_path().string() == "foo" );
-  BOOST_CHECK( p.branch_path().string() == "" );
-  BOOST_CHECK( p.leaf() == "foo" );
+  BOOST_CHECK( p.parent_path().string() == "" );
+  BOOST_CHECK( p.filename() == "foo" );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "" );
   BOOST_CHECK( p.root_path().string() == "" );
@@ -722,14 +636,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "/foo";
   CHECK_EQUAL( p.relative_path().string(), "foo" );
-  CHECK_EQUAL( p.branch_path().string(), "/" );
-  CHECK_EQUAL( p.leaf(), "foo" );
+  CHECK_EQUAL( p.parent_path().string(), "/" );
+  CHECK_EQUAL( p.filename(), "foo" );
   CHECK_EQUAL( p.root_name(), "" );
   CHECK_EQUAL( p.root_directory(), "/" );
   CHECK_EQUAL( p.root_path().string(), "/" );
@@ -737,8 +651,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   if ( platform == "POSIX" )
     BOOST_CHECK( p.is_complete() );
   else
@@ -746,8 +660,8 @@ int test_main( int, char*[] )
 
   p = "/foo/";
   CHECK_EQUAL( p.relative_path().string(), "foo/" );
-  CHECK_EQUAL( p.branch_path().string(), "/foo" );
-  CHECK_EQUAL( p.leaf(), "." );
+  CHECK_EQUAL( p.parent_path().string(), "/foo" );
+  CHECK_EQUAL( p.filename(), "." );
   CHECK_EQUAL( p.root_name(), "" );
   CHECK_EQUAL( p.root_directory(), "/" );
   CHECK_EQUAL( p.root_path().string(), "/" );
@@ -755,8 +669,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   if ( platform == "POSIX" )
     BOOST_CHECK( p.is_complete() );
   else
@@ -764,8 +678,8 @@ int test_main( int, char*[] )
 
   p = "///foo";
   CHECK_EQUAL( p.relative_path().string(), "foo" );
-  CHECK_EQUAL( p.branch_path().string(), "/" );
-  CHECK_EQUAL( p.leaf(), "foo" );
+  CHECK_EQUAL( p.parent_path().string(), "/" );
+  CHECK_EQUAL( p.filename(), "foo" );
   CHECK_EQUAL( p.root_name(), "" );
   CHECK_EQUAL( p.root_directory(), "/" );
   CHECK_EQUAL( p.root_path().string(), "/" );
@@ -773,8 +687,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   if ( platform == "POSIX" )
     BOOST_CHECK( p.is_complete() );
   else
@@ -782,8 +696,8 @@ int test_main( int, char*[] )
 
   p = "foo/bar";
   BOOST_CHECK( p.relative_path().string() == "foo/bar" );
-  BOOST_CHECK( p.branch_path().string() == "foo" );
-  BOOST_CHECK( p.leaf() == "bar" );
+  BOOST_CHECK( p.parent_path().string() == "foo" );
+  BOOST_CHECK( p.filename() == "bar" );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "" );
   BOOST_CHECK( p.root_path().string() == "" );
@@ -791,14 +705,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "../foo";
   BOOST_CHECK( p.relative_path().string() == "../foo" );
-  BOOST_CHECK( p.branch_path().string() == ".." );
-  BOOST_CHECK( p.leaf() == "foo" );
+  BOOST_CHECK( p.parent_path().string() == ".." );
+  BOOST_CHECK( p.filename() == "foo" );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "" );
   BOOST_CHECK( p.root_path().string() == "" );
@@ -806,14 +720,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "..///foo";
   CHECK_EQUAL( p.relative_path().string(), "..///foo" );
-  CHECK_EQUAL( p.branch_path().string(), ".." );
-  CHECK_EQUAL( p.leaf(), "foo" );
+  CHECK_EQUAL( p.parent_path().string(), ".." );
+  CHECK_EQUAL( p.filename(), "foo" );
   CHECK_EQUAL( p.root_name(), "" );
   CHECK_EQUAL( p.root_directory(), "" );
   CHECK_EQUAL( p.root_path().string(), "" );
@@ -821,14 +735,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = "/foo/bar";
   BOOST_CHECK( p.relative_path().string() == "foo/bar" );
-  BOOST_CHECK( p.branch_path().string() == "/foo" );
-  BOOST_CHECK( p.leaf() == "bar" );
+  BOOST_CHECK( p.parent_path().string() == "/foo" );
+  BOOST_CHECK( p.filename() == "bar" );
   BOOST_CHECK( p.root_name() == "" );
   BOOST_CHECK( p.root_directory() == "/" );
   BOOST_CHECK( p.root_path().string() == "/" );
@@ -836,8 +750,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( !p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   if ( platform == "POSIX" )
     BOOST_CHECK( p.is_complete() );
   else
@@ -852,8 +766,8 @@ int test_main( int, char*[] )
   p = path( "//net" );
   CHECK_EQUAL( p.string(), "//net" );
   CHECK_EQUAL( p.relative_path().string(), "" );
-  CHECK_EQUAL( p.branch_path().string(), "" );
-  CHECK_EQUAL( p.leaf(), "//net" );
+  CHECK_EQUAL( p.parent_path().string(), "" );
+  CHECK_EQUAL( p.filename(), "//net" );
   CHECK_EQUAL( p.root_name(), "//net" );
   CHECK_EQUAL( p.root_directory(), "" );
   CHECK_EQUAL( p.root_path().string(), "//net" );
@@ -861,14 +775,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( p.has_root_name() );
   BOOST_CHECK( !p.has_root_directory() );
   BOOST_CHECK( !p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( !p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( !p.has_parent_path() );
   BOOST_CHECK( !p.is_complete() );
 
   p = path( "//net/" );
   BOOST_CHECK( p.relative_path().string() == "" );
-  BOOST_CHECK( p.branch_path().string() == "//net" );
-  BOOST_CHECK( p.leaf() == "/" );
+  BOOST_CHECK( p.parent_path().string() == "//net" );
+  BOOST_CHECK( p.filename() == "/" );
   BOOST_CHECK( p.root_name() == "//net" );
   BOOST_CHECK( p.root_directory() == "/" );
   BOOST_CHECK( p.root_path().string() == "//net/" );
@@ -876,14 +790,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( !p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   BOOST_CHECK( p.is_complete() );
 
   p = path( "//net/foo" );
   BOOST_CHECK( p.relative_path().string() == "foo" );
-  BOOST_CHECK( p.branch_path().string() == "//net/" );
-  BOOST_CHECK( p.leaf() == "foo" );
+  BOOST_CHECK( p.parent_path().string() == "//net/" );
+  BOOST_CHECK( p.filename() == "foo" );
   BOOST_CHECK( p.root_name() == "//net" );
   BOOST_CHECK( p.root_directory() == "/" );
   BOOST_CHECK( p.root_path().string() == "//net/" );
@@ -891,14 +805,14 @@ int test_main( int, char*[] )
   BOOST_CHECK( p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   BOOST_CHECK( p.is_complete() );
 
   p = path( "//net///foo" );
   CHECK_EQUAL( p.relative_path().string(), "foo" );
-  CHECK_EQUAL( p.branch_path().string(), "//net/" );
-  CHECK_EQUAL( p.leaf(), "foo" );
+  CHECK_EQUAL( p.parent_path().string(), "//net/" );
+  CHECK_EQUAL( p.filename(), "foo" );
   CHECK_EQUAL( p.root_name(), "//net" );
   CHECK_EQUAL( p.root_directory(), "/" );
   CHECK_EQUAL( p.root_path().string(), "//net/" );
@@ -906,8 +820,8 @@ int test_main( int, char*[] )
   BOOST_CHECK( p.has_root_name() );
   BOOST_CHECK( p.has_root_directory() );
   BOOST_CHECK( p.has_relative_path() );
-  BOOST_CHECK( p.has_leaf() );
-  BOOST_CHECK( p.has_branch_path() );
+  BOOST_CHECK( p.has_filename() );
+  BOOST_CHECK( p.has_parent_path() );
   BOOST_CHECK( p.is_complete() );
 
   if ( platform == "Windows" )
@@ -959,8 +873,8 @@ int test_main( int, char*[] )
 
     p = path( "c:" );
     BOOST_CHECK( p.relative_path().string() == "" );
-    BOOST_CHECK( p.branch_path().string() == "" );
-    BOOST_CHECK( p.leaf() == "c:" );
+    BOOST_CHECK( p.parent_path().string() == "" );
+    BOOST_CHECK( p.filename() == "c:" );
     BOOST_CHECK( p.root_name() == "c:" );
     BOOST_CHECK( p.root_directory() == "" );
     BOOST_CHECK( p.root_path().string() == "c:" );
@@ -968,14 +882,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( !p.has_root_directory() );
     BOOST_CHECK( !p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( !p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( !p.has_parent_path() );
     BOOST_CHECK( !p.is_complete() );
 
     p = path( "c:foo" );
     BOOST_CHECK( p.relative_path().string() == "foo" );
-    BOOST_CHECK( p.branch_path().string() == "c:" );
-    BOOST_CHECK( p.leaf() == "foo" );
+    BOOST_CHECK( p.parent_path().string() == "c:" );
+    BOOST_CHECK( p.filename() == "foo" );
     BOOST_CHECK( p.root_name() == "c:" );
     BOOST_CHECK( p.root_directory() == "" );
     BOOST_CHECK( p.root_path().string() == "c:" );
@@ -983,14 +897,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( !p.has_root_directory() );
     BOOST_CHECK( p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( !p.is_complete() );
  
     p = path( "c:/" );
     BOOST_CHECK( p.relative_path().string() == "" );
-    BOOST_CHECK( p.branch_path().string() == "c:" );
-    BOOST_CHECK( p.leaf() == "/" );
+    BOOST_CHECK( p.parent_path().string() == "c:" );
+    BOOST_CHECK( p.filename() == "/" );
     BOOST_CHECK( p.root_name() == "c:" );
     BOOST_CHECK( p.root_directory() == "/" );
     BOOST_CHECK( p.root_path().string() == "c:/" );
@@ -998,14 +912,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( p.has_root_directory() );
     BOOST_CHECK( !p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( p.is_complete() );
 
     p = path( "c:.." );
     BOOST_CHECK( p.relative_path().string() == ".." );
-    BOOST_CHECK( p.branch_path().string() == "c:" );
-    BOOST_CHECK( p.leaf() == ".." );
+    BOOST_CHECK( p.parent_path().string() == "c:" );
+    BOOST_CHECK( p.filename() == ".." );
     BOOST_CHECK( p.root_name() == "c:" );
     BOOST_CHECK( p.root_directory() == "" );
     BOOST_CHECK( p.root_path().string() == "c:" );
@@ -1013,14 +927,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( !p.has_root_directory() );
     BOOST_CHECK( p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( !p.is_complete() );
 
     p = path( "c:/foo" );
     CHECK_EQUAL( p.relative_path().string(), "foo" );
-    CHECK_EQUAL( p.branch_path().string(), "c:/" );
-    CHECK_EQUAL( p.leaf(), "foo" );
+    CHECK_EQUAL( p.parent_path().string(), "c:/" );
+    CHECK_EQUAL( p.filename(), "foo" );
     CHECK_EQUAL( p.root_name(), "c:" );
     CHECK_EQUAL( p.root_directory(), "/" );
     CHECK_EQUAL( p.root_path().string(), "c:/" );
@@ -1028,14 +942,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( p.has_root_directory() );
     BOOST_CHECK( p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( p.is_complete() );
 
     p = path( "c://foo" );
     CHECK_EQUAL( p.relative_path().string(), "foo" );
-    CHECK_EQUAL( p.branch_path().string(), "c:/" );
-    CHECK_EQUAL( p.leaf(), "foo" );
+    CHECK_EQUAL( p.parent_path().string(), "c:/" );
+    CHECK_EQUAL( p.filename(), "foo" );
     CHECK_EQUAL( p.root_name(), "c:" );
     CHECK_EQUAL( p.root_directory(), "/" );
     CHECK_EQUAL( p.root_path().string(), "c:/" );
@@ -1043,14 +957,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( p.has_root_directory() );
     BOOST_CHECK( p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( p.is_complete() );
 
     p = path( "c:\\foo\\bar" );
     CHECK_EQUAL( p.relative_path().string(), "foo/bar" );
-    CHECK_EQUAL( p.branch_path().string(), "c:/foo" );
-    CHECK_EQUAL( p.leaf(), "bar" );
+    CHECK_EQUAL( p.parent_path().string(), "c:/foo" );
+    CHECK_EQUAL( p.filename(), "bar" );
     CHECK_EQUAL( p.root_name(), "c:" );
     CHECK_EQUAL( p.root_directory(), "/" );
     CHECK_EQUAL( p.root_path().string(), "c:/" );
@@ -1058,14 +972,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( p.has_root_directory() );
     BOOST_CHECK( p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( p.is_complete() );
 
     p = path( "prn:" );
     BOOST_CHECK( p.relative_path().string() == "" );
-    BOOST_CHECK( p.branch_path().string() == "" );
-    BOOST_CHECK( p.leaf() == "prn:" );
+    BOOST_CHECK( p.parent_path().string() == "" );
+    BOOST_CHECK( p.filename() == "prn:" );
     BOOST_CHECK( p.root_name() == "prn:" );
     BOOST_CHECK( p.root_directory() == "" );
     BOOST_CHECK( p.root_path().string() == "prn:" );
@@ -1073,14 +987,14 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( !p.has_root_directory() );
     BOOST_CHECK( !p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( !p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( !p.has_parent_path() );
     BOOST_CHECK( !p.is_complete() );
 
     p = path( "\\\\net\\\\\\foo" );
     CHECK_EQUAL( p.relative_path().string(), "foo" );
-    CHECK_EQUAL( p.branch_path().string(), "//net/" );
-    CHECK_EQUAL( p.leaf(), "foo" );
+    CHECK_EQUAL( p.parent_path().string(), "//net/" );
+    CHECK_EQUAL( p.filename(), "foo" );
     CHECK_EQUAL( p.root_name(), "//net" );
     CHECK_EQUAL( p.root_directory(), "/" );
     CHECK_EQUAL( p.root_path().string(), "//net/" );
@@ -1088,8 +1002,8 @@ int test_main( int, char*[] )
     BOOST_CHECK( p.has_root_name() );
     BOOST_CHECK( p.has_root_directory() );
     BOOST_CHECK( p.has_relative_path() );
-    BOOST_CHECK( p.has_leaf() );
-    BOOST_CHECK( p.has_branch_path() );
+    BOOST_CHECK( p.has_filename() );
+    BOOST_CHECK( p.has_parent_path() );
     BOOST_CHECK( p.is_complete() );
 
     itr_ck = path( "c:" );
@@ -1283,6 +1197,40 @@ int test_main( int, char*[] )
   BOOST_CHECK( acs2 >= a );
   BOOST_CHECK( a2 >= as );
   BOOST_CHECK( a2 >= acs );
+  
+// extension() tests
+
+  BOOST_CHECK( path("a/b").extension() == "" );
+  BOOST_CHECK( path("a/b.txt").extension() == ".txt" );
+  BOOST_CHECK( path("a/b.").extension() == "." );
+  BOOST_CHECK( path("a.b.c").extension() == ".c" );
+  BOOST_CHECK( path("a.b.c.").extension() == "." );
+  BOOST_CHECK( path("").extension() == "" );
+  BOOST_CHECK( path("a/").extension() == "." );
+  
+// stem() tests
+
+  BOOST_CHECK( path("b").stem() == "b" );
+  BOOST_CHECK( path("a/b.txt").stem() == "b" );
+  BOOST_CHECK( path("a/b.").stem() == "b" ); 
+  BOOST_CHECK( path("a.b.c").stem() == "a.b" );
+  BOOST_CHECK( path("a.b.c.").stem() == "a.b.c" );
+  BOOST_CHECK( path("").stem() == "" );
+  
+// replace_extension() tests
+
+  BOOST_CHECK( path("a.txt").replace_extension("").string() == "a" );
+  BOOST_CHECK( path("a.txt").replace_extension(".").string() == "a." );
+  BOOST_CHECK( path("a.txt").replace_extension(".tex").string() == "a.tex" );
+  BOOST_CHECK( path("a.txt").replace_extension("tex").string() == "a.tex" );
+  BOOST_CHECK( path("a.").replace_extension(".tex").string() == "a.tex" );
+  BOOST_CHECK( path("a.").replace_extension("tex").string() == "a.tex" );
+  BOOST_CHECK( path("a").replace_extension(".txt").string() == "a.txt" );
+  BOOST_CHECK( path("a").replace_extension("txt").string() == "a.txt" );
+  BOOST_CHECK( path("a.b.txt" ).replace_extension(".tex").string() == "a.b.tex" );  
+  BOOST_CHECK( path("a.b.txt" ).replace_extension("tex").string() == "a.b.tex" );  
+  // see the rationale in html docs for explanation why this works
+  BOOST_CHECK( path("").replace_extension(".png").string() == ".png" );
 
   // inserter and extractor tests
 # if !defined( BOOST_MSVC ) || BOOST_MSVC > 1300 // bypass VC++ 7.0 and earlier
