@@ -13,10 +13,44 @@
 #include <boost/ptr_container/exception.hpp>
 #include <boost/range/sub_range.hpp>
 
-template< typename C, typename B, typename T >
+template< typename C, typename B, typename T, bool Ordered >
 void ptr_set_test();
 
-template< typename C, typename B, typename T >
+template< class T, bool Ordered >
+struct test_algorithms
+{
+    template< class Cont >
+    void operator()( Cont& c, const Cont& c2 ) const
+    {
+        typename Cont::iterator i;
+        typename Cont::const_iterator ci;
+
+        T* t = new T;
+        i  = c.lower_bound( *t );
+        ci = c2.lower_bound( *t );
+        i  = c.upper_bound( *t );
+        ci = c2.upper_bound( *t );
+        delete t;
+
+        BOOST_DEDUCED_TYPENAME Cont::reverse_iterator ri         = c.rbegin();
+        BOOST_DEDUCED_TYPENAME Cont::const_reverse_iterator cri  = c2.rbegin();
+        BOOST_DEDUCED_TYPENAME Cont::reverse_iterator rv2        = c.rend();
+        BOOST_DEDUCED_TYPENAME Cont::const_reverse_iterator cvr2 = c2.rend();
+        cri = c.crbegin();
+        cri = c.crend();
+    }
+};
+
+template< class T >
+struct test_algorithms<T,false>
+{
+    template< class Cont>
+    void operator()( Cont& c, const Cont& c2 ) const
+    {
+    }
+};
+
+template< typename C, typename B, typename T, bool Ordered >
 void ptr_set_test()
 {   
     using namespace boost;
@@ -27,7 +61,7 @@ void ptr_set_test()
     BOOST_CHECK( c.size() == 0 );
     c.insert( c.end(), new T );
     c.insert( c.end(), new T );
-
+    
     const C c2( c.begin(), c.end() );
     BOOST_CHECK( c.size() == c2.size() );
     
@@ -49,12 +83,10 @@ void ptr_set_test()
     BOOST_DEDUCED_TYPENAME C::allocator_type alloc        = c.get_allocator();
     BOOST_DEDUCED_TYPENAME C::iterator i                  = c.begin();
     BOOST_DEDUCED_TYPENAME C::const_iterator ci           = c2.begin();
+    ci = c.cbegin();
+    ci = c.cend();
     BOOST_DEDUCED_TYPENAME C::iterator i2                 = c.end();
     BOOST_DEDUCED_TYPENAME C::const_iterator ci2          = c2.begin();
-    BOOST_DEDUCED_TYPENAME C::reverse_iterator ri         = c.rbegin();
-    BOOST_DEDUCED_TYPENAME C::const_reverse_iterator cri  = c2.rbegin();
-    BOOST_DEDUCED_TYPENAME C::reverse_iterator rv2        = c.rend();
-    BOOST_DEDUCED_TYPENAME C::const_reverse_iterator cvr2 = c2.rend();
                              
     BOOST_MESSAGE( "finished iterator test" ); 
    
@@ -111,6 +143,7 @@ void ptr_set_test()
     BOOST_CHECK( c3.empty() == false );
     c.clear();
     unsigned long c3size = c3.size();
+    hide_warning( c3size );
     unsigned long num  = c. BOOST_NESTED_TEMPLATE transfer<C>( c3.begin(), 
                                                                c3.end(), 
                                                                c3 );
@@ -142,14 +175,12 @@ void ptr_set_test()
     i  = c.find( *t );
     ci = c2.find( *t );
     c2.count( *t );
-    i  = c.lower_bound( *t );
-    ci = c2.lower_bound( *t );
-    i  = c.upper_bound( *t );
-    ci = c2.upper_bound( *t );
+
+    test_algorithms<T,Ordered>()( c, c2 );
     sub  = c.equal_range( *t );
     csub = c2.equal_range( *t );         
     delete t;
-    
+       
     BOOST_MESSAGE( "finished algorithms interface test" );         
     
 }
