@@ -47,7 +47,7 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 //  import required names
-using namespace boost::spirit;
+using namespace boost::spirit::classic;
 
 using std::string;
 using std::pair;
